/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package tabs;

import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.util.ResourceBundle;


import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.TableItem;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import core.Loader;

import tabs.hair.HairButton;
import tabs.hair.HairColorData;
import tabs.hair.HairColoredSample;
import tabs.hair.HairStyleData;
import tabs.hair.HairStyleXMLHandler;
import utils.ImageUtils;

/**
 * Instances of this class represents a tab that
 * can be used to draw an hair style over the
 * drawing's area image
 *
 */
public class HairTab extends TabContainer {

	/**
	 * Instances of this class represents a state of the {@link HairTab}
	 */
	private class HairHistoryStep {

		private final java.awt.Point finalImageOffset;

		/**
		 * The selected button
		 */
		private final HairButton selectedHairButton;

		/**
		 * The hair color
		 */
		private final int hairColorChooserIndex;

		/**
		 * The highlights color
		 */
		private final int highlightsColorChooserIndex;

		/**
		 * The hair size
		 */
		private final int hairSizeScaleValue;

		/**
		 * Creates a new instance of this class to save the
		 * {@link HairTab} given state
		 * @param finalImageOffset hair's (x,y) offset
		 * @param selectedHairButton the selected hair button
		 * @param hairColorChooserIndex the hair color
		 * @param highlightsColorChooserIndex the highlights color
		 * @param hairSizeScaleValue the hair's size
		 */
		public HairHistoryStep(java.awt.Point finalImageOffset, HairButton selectedHairButton, int hairColorChooserIndex, int highlightsColorChooserIndex, int hairSizeScaleValue) {

			//create a new point with the values so that we don't get only the reference to finalImageOffset point
			this.finalImageOffset = new java.awt.Point(finalImageOffset);

			//search for the reference of the actual button instead of the reference of the selectedButton
			if (selectedHairButton != null) {
				for (Control c : selectedHairButton.getParent().getChildren()) {
					if (c instanceof HairButton) {
						HairButton hb = (HairButton) c;
						if (hb.equals(selectedHairButton)) {
							selectedHairButton = hb;
							break;
						}
					}
				}
			}
			this.selectedHairButton = selectedHairButton;
			this.hairColorChooserIndex = hairColorChooserIndex;
			this.highlightsColorChooserIndex = highlightsColorChooserIndex;
			this.hairSizeScaleValue = hairSizeScaleValue;
		}

	}

	//width and height for the hair button
	private static final int HAIR_BUTTON_WIDTH = 55;
	private static final int HAIR_BUTTON_HEIGHT = 75;

	//width and height for the gender buttons
	private static final int GENDER_COLOR_BUTTON_WIDTH = 26;
	private static final int GENDER_COLOR_BUTTON_HEIGHT = 28;

	//colors for the female/male gender buttons
	private static final Color FEMALE_COLOR = new Color(Display.getCurrent(), 236, 173, 225);
	private static final Color MALE_COLOR = new Color(Display.getCurrent(), 101, 148, 209);

	/**
	 * Used to set the upper composite
	 */
	private TabGuiUtility tabGui = null;

	/**
	 * the tab's main composite
	 */
	private Composite mainComposite;

	/**
	 * Composite used to draw all hair controls
	 * (gender buttons, hair type combo, hair buttons)
	 */
	private Composite hairsComposite = null;

	/**
	 * Composite used as a base for all hair buttons
	 */
	private ScrolledComposite scrolledComposite = null;

	/**
	 * Composite used to hold the various hair type groups buttons
	 */
	private Composite scrolledCompositeContainer = null;

	/**
	 * Composite for the bottom controls
	 */
	private Composite bottomComposite = null;

	//Separator labels for the bottom composite
	private Label labelSeparatorUp = null;
	private Label labelSeparatorDown = null;

	/**
	 * Scale for the size of the hair image
	 */
	private Scale sizeScale = null;

	/**
	 * label for the size scale
	 */
	private Label labelSizeScale = null;

	/**
	 * ComboBox for choosing the desired hair types
	 */
	private Combo hairTypesCombo = null;

	/**
	 * Label for the hair types ComboBox
	 */
	private Label labelHairTypeCombo = null;

	//color chooser buttons for the hair and highlights
	private ColorChooserButton hairColorChooserButton;
	private ColorChooserButton highlightsColorChooserButton;

	//labels for the color chooser buttons
	private Label labelHairColor = null;
	private Label labelHighlightsColor = null;

	//male and female color buttons and a pointer to the current selected gender button
	private ColorButton femaleColorButton = null;
	private ColorButton maleColorButton = null;
	private ColorButton selectedGender = null;

	/**
	 * Pointer to the current selected hair button
	 */
	private HairButton selectedHairButton = null;

	//TODO a better implementation needed here
	private BufferedImage originalHairImage = null;
	private BufferedImage originalHighlitesImage = null;
	private BufferedImage originalShadowImage = null;
	private BufferedImage finalImage = null;

	/**
	 * the current image offset for this tab's image in the drawing area
	 */
	private java.awt.Point hairTabImageOffset = null;

	/**
	 * The XML reader used by this hair tab instance to parse the XML hair style files
	 */
	private XMLReader parser = null;

	//used to help with the dragging of the hair image over the drawing area
	private int startX;
	private int startY;
	private java.awt.Point oldImageOffset;

	/**
	 * flag to indicate if the mouse dragging the hair in the project's drawing area
	 */
	//TODO check if other tabs that support object dragging needs this feature
	private boolean mouseIsDragged;

	/**
	 * The XML handler used by this hair tab instance
	 */
	private HairStyleXMLHandler hairStyleHandler;

	/**
	 * messages that are to be used by the entire GUI
	 */
	private ResourceBundle messages;

	//hold the data of the current selected hair color and highlights color
	private HairColorData selectedHairColor = null;
	private HairColorData selectedHighlightsColor = null;

	//mouse listeners in the drawing area
	private java.awt.event.MouseListener mouseListener;
	private MouseMotionListener mouseMotionListener;

	/**
	 * flag to indicate whether the tab has or not access to call for redrawing the image area
	 */
	private boolean imageAreaRedrawAllowed;

	/**
	 * flag to indicate whether the scaling should do a quality (slower) scale or a fast one
	 */
	protected boolean doQualityScale;

	/**
	 * A reference to this instance class
	 */
	private HairTab thisHairTab = this;

	/**
	 * flag used to indicate whether the hair tab should draw it's hair image in the drawing area
	 */
	private boolean HAIR_VISIBILITY = true;

	/**
	 * Constructs a new instance of this class given its parent,
	 * a style value describing its behavior and appearance and
	 * this instance's position in the {@link TabsContainer} array
	 *
	 * @param parent a {@link TabsContainer} which will be the parent of the new instance
	 * @param style the style of the tab
	 * @param tabLayerPosition the tab's position in the {@link TabsContainer} array
	 *
	 */
	public HairTab(TabsContainer parent, int style, int tabLayerPosition) {
		super(parent, style, tabLayerPosition);

		messages = Loader.getMessages();

		//the 2 things needed to be done by every implementation of TabComposite
		super.setTabImage(new Image(Display.getCurrent(), System.getProperty("program.pixmapspath") + "tab_icon_hair.png"));
		super.setTabToolTipText(messages.getString("HAIR_TAB_TOOLTIP"));

		hairTabImageOffset = new java.awt.Point(0,0);

		//create upper composite and initialize the mainComposite
		tabGui = new TabGuiUtility(this);
		mainComposite = this;
		tabGui.createUpperComposite(super.getTabToolTipText());

		initXMLSupport();

		//create hairs container
		createHairsComposite();

		//create bottom composite
		createBottomComposite();

		imageAreaRedrawAllowed = true;

	}

	/**
	 * initialize the XML reader and XML handler used for parsing
	 */
	private void initXMLSupport() {
		//TODO this code here!?
		try {
			parser = XMLReaderFactory.createXMLReader();
		} catch (SAXException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		hairStyleHandler = new HairStyleXMLHandler();
	}

	/**
	 * This method initializes hairsComposite
	 */
	private void createHairsComposite() {

		//grid data for the composite
		GridData gridDataComposite = new GridData();
		gridDataComposite.grabExcessHorizontalSpace = true;
		gridDataComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.CENTER;
		gridDataComposite.grabExcessVerticalSpace = true;
		gridDataComposite.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//grid layout for the composite
		GridLayout gridLayoutComposite = new GridLayout();
		gridLayoutComposite.numColumns = 4;

		//initialize the composite
		hairsComposite = new Composite(mainComposite, SWT.NONE);
		hairsComposite.setLayout(gridLayoutComposite);
		hairsComposite.setLayoutData(gridDataComposite);

		//grid data for the two gender buttons
		GridData gridDataGenderButtons = new GridData();
		gridDataGenderButtons.widthHint = GENDER_COLOR_BUTTON_WIDTH;
		gridDataGenderButtons.heightHint = GENDER_COLOR_BUTTON_HEIGHT;

		//initialize the female color button
		femaleColorButton = new ColorButton(hairsComposite, SWT.NONE);
		femaleColorButton.setBackground(FEMALE_COLOR);
		femaleColorButton.setBackgroundImage(new Image(Display.getCurrent(), System.getProperty("program.pixmapspath") + "tab_hair_women.png"));
		femaleColorButton.setLayoutData(gridDataGenderButtons);
		femaleColorButton.setData("gender", "female");

		//initialize the male color button
		maleColorButton = new ColorButton(hairsComposite, SWT.NONE);
		maleColorButton.setBackground(MALE_COLOR);
		maleColorButton.setBackgroundImage(new Image(Display.getCurrent(), System.getProperty("program.pixmapspath") + "tab_hair_men.png"));
		maleColorButton.setLayoutData(gridDataGenderButtons);
		maleColorButton.setData("gender", "male");

		//select default gender button
		selectedGender = femaleColorButton;
		femaleColorButton.setSelected(true);

		//gender buttons listeners
		femaleColorButton.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
			public void mouseDown(org.eclipse.swt.events.MouseEvent e) {
				//if the clicked genderColorButton is not selected, select it
				if (!selectedGender.equals(femaleColorButton)) {
					selectedGender.setSelected(false);
					selectedGender = femaleColorButton;
					selectedGender.setSelected(true);
					//change hair list...
					Event evt = new Event();
					evt.widget = hairTypesCombo;
					hairTypesCombo.notifyListeners(SWT.Selection, evt);

				}
			}
		});
		maleColorButton.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
			public void mouseDown(org.eclipse.swt.events.MouseEvent e) {
				//if the clicked genderColorButton is not selected, select it
				if (!selectedGender.equals(maleColorButton)) {
					selectedGender.setSelected(false);
					selectedGender = maleColorButton;
					selectedGender.setSelected(true);
					//change hair list...
					Event evt = new Event();
					evt.widget = hairTypesCombo;
					hairTypesCombo.notifyListeners(SWT.Selection, evt);
				}
			}
		});

		//initialize the hair type combo label
		labelHairTypeCombo = new Label(hairsComposite, SWT.NONE);
		labelHairTypeCombo.setText(messages.getString("HAIR_TAB_HAIRTYPE"));

		//initialize the ComboBox used for choosing an hair style type
		hairTypesCombo = new Combo(hairsComposite, SWT.READ_ONLY);
		hairTypesCombo.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				if (e.widget instanceof Combo) {
					Combo c = (Combo) e.widget;
					Composite hairButtonsComposite = (Composite) c.getData(c.getItem(c.getSelectionIndex()) + selectedGender.getData("gender"));
					changeHairButtonsComposite(hairButtonsComposite);					
				}
			}
		});

		//hair list
		createHairsScrolledContainer();

	}

	/**
	 * This method initialize the scrolled composite, it's container
	 * and create all hair button type groups each with it's hair buttons
	 */
	private void createHairsScrolledContainer() {

		//grid data for the scrolled comoposite
		GridData gridDataScrolledComposite = new GridData();
		gridDataScrolledComposite.horizontalSpan = 4;
		gridDataScrolledComposite.grabExcessVerticalSpace = true;
		gridDataScrolledComposite.verticalIndent = 2;
		gridDataScrolledComposite.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridDataScrolledComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;		

		//grid data for the scrolled composite container
		GridData gridDataScrolledCompositeContainer = new GridData();
		gridDataScrolledCompositeContainer.horizontalAlignment = org.eclipse.swt.layout.GridData.CENTER;
		gridDataScrolledCompositeContainer.grabExcessHorizontalSpace = true;

		//initialize the scrolled composite
		scrolledComposite = new ScrolledComposite(hairsComposite, SWT.BORDER | SWT.V_SCROLL);
		scrolledComposite.setAlwaysShowScrollBars(true);
		scrolledComposite.setLayoutData(gridDataScrolledComposite);
		scrolledComposite.getVerticalBar().setIncrement(HAIR_BUTTON_HEIGHT);
		scrolledComposite.getVerticalBar().setPageIncrement(3 * HAIR_BUTTON_HEIGHT);
		scrolledComposite.setExpandVertical(true);
		scrolledComposite.setExpandHorizontal(true);

		//initialize the scrolled composite container
		scrolledCompositeContainer = new Composite(scrolledComposite,SWT.NONE);
		scrolledCompositeContainer.setLayoutData(gridDataScrolledCompositeContainer);
		scrolledCompositeContainer.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));

		//set the scrolled composite container to be the scrolled composite's content
		scrolledComposite.setContent(scrolledCompositeContainer);

		//grid layout for the hair buttons composite
		GridLayout gridLayoutHairButtonsComposite = new GridLayout();
		gridLayoutHairButtonsComposite.numColumns = 3;

		//grid data for the hair buttons
		GridData gridDataHairButton = new GridData();
		gridDataHairButton.widthHint = HAIR_BUTTON_WIDTH;
		gridDataHairButton.heightHint = HAIR_BUTTON_HEIGHT;

		//hair buttons listeners
		MouseAdapter mouseListener = new org.eclipse.swt.events.MouseAdapter() {
			public void mouseDown(org.eclipse.swt.events.MouseEvent e) {
				setFocus();
				Control child = (Control)e.widget;
				adjustScrollPosition(child);
				tabsContainer.createHistoryStep();
			}

			public void mouseUp(org.eclipse.swt.events.MouseEvent e) {
				if (selectedHairButton != null) {
					selectedHairButton.setSelected(false);
				}

				selectedHairButton = (HairButton) e.widget;

				//check if there already exists a hair color and highlights color for the hair
				//if not select by default the first entry
				if (hairColorChooserButton.getSelectionIndex() == -1) {
					hairColorChooserButton.select(0); //the first color selected by default
				}
				if (highlightsColorChooserButton.getSelectionIndex() == -1) {
					highlightsColorChooserButton.select(0); //the first color selected by default
				}

				//loosing 68-225 milliseconds
				BufferedImage[] images = ((HairButton)e.widget).getImages(hairStyleHandler,parser);
				originalHairImage = images[0];
				originalHighlitesImage = images[1];
				originalShadowImage = images[2];

				//select the new hair button and enable the bottom controls
				selectedHairButton.setSelected(true);
				setBottomCompositeEnabled(true);

				//redraw the tab's image
				redrawHairTabImage();
			}
		};

		//add all hair types to combo and create the composite for the type (male copmposite and female composite) 
		for (int i = 0; i < messages.getString("COMBO_ITEMS_ORDERED").split(",").length; i++) {

			//get the hair type and add the translation text to combo
			String comboItem = messages.getString("COMBO_ITEMS_ORDERED").split(",")[i].trim();
			hairTypesCombo.add(messages.getString(comboItem));

			//create the male composite for this hair type
			Composite hairButtonsCompositeMale = new Composite(scrolledCompositeContainer, SWT.NONE);
			hairButtonsCompositeMale.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
			hairButtonsCompositeMale.setLayout(gridLayoutHairButtonsComposite);
			hairButtonsCompositeMale.setData("gender", maleColorButton.getData("gender"));
			hairButtonsCompositeMale.setData("comboIndex", hairTypesCombo.getItemCount() - 1);
			hairButtonsCompositeMale.setVisible(false);

			//create the female composite for this hair type
			Composite hairButtonsCompositeFemale = new Composite(scrolledCompositeContainer, SWT.NONE);
			hairButtonsCompositeFemale.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
			hairButtonsCompositeFemale.setLayout(gridLayoutHairButtonsComposite);
			hairButtonsCompositeFemale.setData("gender", femaleColorButton.getData("gender"));
			hairButtonsCompositeFemale.setData("comboIndex", hairTypesCombo.getItemCount() - 1);
			hairButtonsCompositeFemale.setVisible(false);

			//add male and female composite to combo's data so that it can be accessed later
			//(the key is created by using the translated type and gender together)
			hairTypesCombo.setData(messages.getString(comboItem) + maleColorButton.getData("gender"), hairButtonsCompositeMale);
			hairTypesCombo.setData(messages.getString(comboItem) + femaleColorButton.getData("gender"), hairButtonsCompositeFemale);

		}

		//create a hair button for every hair style found in loader
		for (HairStyleData hairStyle : Loader.hairThumbnail) {

			//add this hairStyle to it's respective position in combo
			Composite hairStypeParentComposite = (Composite) hairTypesCombo.getData(messages.getString(hairStyle.getHairType()) + hairStyle.getGender());

			//create a hair button for this hair style
			HairButton hairStyleButton = new HairButton(hairStypeParentComposite, SWT.NONE);
			hairStyleButton.setBackgroundImage(hairStyle.getThumbnail());
			hairStyleButton.setAssociatedXML(hairStyle.getHairStyleFile());
			hairStyleButton.setLayoutData(gridDataHairButton);
			hairStyleButton.setCursor(Display.getCurrent().getSystemCursor(SWT.CURSOR_HAND));
			hairStyleButton.addMouseListener(mouseListener);
			hairStyleButton.setToolTipText(hairStyle.getHairStyleName());

		}

		//select the default hair type
		hairTypesCombo.select(getDefaultHairType());
		Event evt = new Event();
		evt.widget = hairTypesCombo;
		hairTypesCombo.notifyListeners(SWT.Selection, evt);
	}

	/**
	 * Every time the hair buttons visible group is changed (by changing hair type or gender)
	 * scrolled composite needs to adjust it's vertical scroll
	 * @param child the new visible hair buttons group
	 */
	private void adjustScrollPosition (Control child) {
		Rectangle bounds = child.getBounds();
		bounds.height += 10;
		bounds.y -= 5;
		Rectangle area = scrolledComposite.getClientArea();
		Point origin = scrolledComposite.getOrigin();
		if (origin.x > bounds.x) origin.x = Math.max(0, bounds.x);
		if (origin.y > bounds.y) origin.y = Math.max(0, bounds.y);
		if (origin.x + area.width < bounds.x + bounds.width) origin.x = Math.max(0, bounds.x + bounds.width - area.width);
		if (origin.y + area.height < bounds.y + bounds.height) origin.y = Math.max(0, bounds.y + bounds.height - area.height);
		scrolledComposite.setOrigin(origin);
	}

	/**
	 * This method redraws the tab's image using the latest hair info selected from controls
	 */
	private void redrawHairTabImage() {

		//loosing 45-120 milliseconds
		if (selectedHairColor != null) { //show the hair on the image only if already exists a color selected for it
			java.awt.Composite compositeToUse = Loader.getCompositeFor(selectedHairColor.getBlendingMode());
			finalImage = new BufferedImage(originalHairImage.getWidth(), originalHairImage.getHeight(), BufferedImage.TYPE_INT_ARGB);
			Graphics2D g2d = finalImage.createGraphics();
			g2d.drawImage(originalShadowImage, 0, 0, null);
			BufferedImage currentHairImage = ImageUtils.applyBlending(originalHairImage, compositeToUse, selectedHairColor.getRed(), selectedHairColor.getGreen(), selectedHairColor.getBlue());
			g2d.drawImage(currentHairImage, 0, 0, null);
			if (selectedHighlightsColor != null) {
				BufferedImage currentHighlitesImage = ImageUtils.applyBlending(originalHighlitesImage, compositeToUse, selectedHighlightsColor.getRed(), selectedHighlightsColor.getGreen(), selectedHighlightsColor.getBlue());
				g2d.drawImage(currentHighlitesImage, 0, 0, null);
			}
			g2d.dispose();
		}

		//loosing 37-96 milliseconds
		if (finalImage != null) {
			double widthFactor = sizeScale.getSelection() / 50.0;
			double heightFactor = sizeScale.getSelection() / 50.0;
			AffineTransform tx = new AffineTransform(); //use affine transform class to do the resizing
			tx.scale(widthFactor, heightFactor);
			AffineTransformOp op = null;
			if (doQualityScale) {
				op = new AffineTransformOp(tx, AffineTransformOp.TYPE_BICUBIC);
			} else {
				op = new AffineTransformOp(tx, AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
			}
			finalImage = op.filter(finalImage, null);
		}
		if (imageAreaRedrawAllowed) {
			tabsContainer.redrawImageArea(thisHairTab);
		}
	}

	/**
	 * Enables the bottom composite controls if the argument is
	 * <code>true</code>, and disables it otherwise. A disabled
	 * control is typically not selectable from the user interface
	 * and draws with an inactive or "grayed" look.
	 *
	 * @param option the new enabled state
	 *
	 * @exception SWTException <ul>
	 *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
	 * </ul>
	 */
	private void setBottomCompositeEnabled(boolean option) {
		labelSizeScale.setEnabled(option);
		sizeScale.setEnabled(option);
		labelHairColor.setEnabled(option);
		hairColorChooserButton.setEnabled(option);
		labelHighlightsColor.setEnabled(option);
		highlightsColorChooserButton.setEnabled(option);
	}

	/**
	 * This method changes the current visible hair buttons group
	 * in the scrolled composite container
	 * @param hairButtonsComposite the new hair buttons group to be visible
	 */
	private void changeHairButtonsComposite(Composite hairButtonsComposite) {
		//hide all hairButtons group composites
		for (Control hairButtonsParent : scrolledCompositeContainer.getChildren()) {
			hairButtonsParent.setVisible(false);
		}
		//show only the desired group
		scrolledComposite.setMinSize(0, hairButtonsComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
		hairButtonsComposite.setVisible(true);
		hairButtonsComposite.pack();
		scrolledComposite.getVerticalBar().setSelection(0);
		scrolledComposite.setOrigin(0,0);
	}

	/**
	 * This method initializes bottomComposite
	 */
	public void createBottomComposite() {

		//grid data for the separator label up
		GridData gridDataSeparatorLabelUp = new GridData();
		gridDataSeparatorLabelUp.grabExcessHorizontalSpace = true;
		gridDataSeparatorLabelUp.horizontalSpan = 2;
		gridDataSeparatorLabelUp.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//grid data for the separator label down
		GridData gridDataSeparatorLabelDown = new GridData();
		gridDataSeparatorLabelDown.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridDataSeparatorLabelDown.horizontalSpan = 2;
		gridDataSeparatorLabelDown.grabExcessHorizontalSpace = true;

		//grid data for the size scale
		GridData gridDataSizeScale = new GridData();
		gridDataSizeScale.grabExcessHorizontalSpace = true;
		gridDataSizeScale.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//grid data for the composite
		GridData gridDataComposite = new GridData();
		gridDataComposite.verticalAlignment = org.eclipse.swt.layout.GridData.END;
		gridDataComposite.grabExcessHorizontalSpace = true;
		gridDataComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//grid layout for the composite
		GridLayout gridLayoutComposite = new GridLayout();
		gridLayoutComposite.numColumns = 2;

		//initialize the composite
		bottomComposite = new Composite(mainComposite, SWT.NONE);
		bottomComposite.setLayout(gridLayoutComposite);
		bottomComposite.setLayoutData(gridDataComposite);

		//initialize the upper label separator
		labelSeparatorUp = new Label(bottomComposite, SWT.SEPARATOR | SWT.HORIZONTAL);
		labelSeparatorUp.setLayoutData(gridDataSeparatorLabelUp);

		//initialize the hair color label
		labelHairColor = new Label(bottomComposite, SWT.HORIZONTAL);
		labelHairColor.setText(messages.getString("HAIR_TAB_HAIRCOLOR"));
		labelHairColor.setEnabled(false);

		//initialize the hair color button
		createHairColorChooserButton();
		hairColorChooserButton.setEnabled(false);

		//initialize the highlights color label
		labelHighlightsColor = new Label(bottomComposite, SWT.HORIZONTAL);
		labelHighlightsColor.setText(messages.getString("HAIR_TAB_HIGHLITESCOLOR"));
		labelHighlightsColor.setEnabled(false);

		//initialize the highlights color button
		createHighlightsColorChooserButton();
		highlightsColorChooserButton.setEnabled(false);

		//initialize the size scale label
		labelSizeScale = new Label(bottomComposite, SWT.NONE);
		labelSizeScale.setText(messages.getString("HAIR_TAB_HAIRSIZE"));
		labelSizeScale.setEnabled(false);

		//initialize the size scale
		sizeScale = new Scale(bottomComposite, SWT.NONE);
		sizeScale.setSelection(50);
		sizeScale.setMinimum(46);
		sizeScale.setMaximum(54);
		sizeScale.setToolTipText("100 %");
		sizeScale.setLayoutData(gridDataSizeScale);
		sizeScale.setEnabled(false);

		//initialize the lower separator label
		labelSeparatorDown = new Label(bottomComposite, SWT.SEPARATOR | SWT.HORIZONTAL);
		labelSeparatorDown.setLayoutData(gridDataSeparatorLabelDown);

		//size scale listeners
		sizeScale.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				if (e.widget instanceof Scale) {
					Scale s = (Scale) e.widget;
					s.setToolTipText((s.getSelection() + 50) + " %");
					redrawHairTabImage();
				}
			}
		});
		sizeScale.addMouseListener(new MouseAdapter() {
			public void mouseDown(org.eclipse.swt.events.MouseEvent e) {
				//TODO history step should be created when scale is changed with keyboard keys
				tabsContainer.createHistoryStep();
				doQualityScale = false;
			}
			public void mouseUp(org.eclipse.swt.events.MouseEvent e) {
				doQualityScale = true;
				redrawHairTabImage();
			}
		});
	}

	/**
	 * This class initializes hairColorChooserButton
	 */
	private void createHairColorChooserButton() {

		//TODO it should exist a method that generates text column width value by
		//getting the larger text's size (because of language files)
		int[] tabColumns = { 67, 200 };

		//grid data for the hair color chooser button
		GridData gridDataHairColorChooser = new GridData();
		gridDataHairColorChooser.grabExcessHorizontalSpace = true;
		gridDataHairColorChooser.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//initialize the hair color chooser button
		hairColorChooserButton = new ColorChooserButton(bottomComposite, tabColumns);
		hairColorChooserButton.setLayoutData(gridDataHairColorChooser);

		//hair color chooser listeners
		hairColorChooserButton.addTableSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				selectedHairColor = (HairColorData)((TableItem)e.item).getData();
				redrawHairTabImage();
			}
		});
		hairColorChooserButton.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
			public void mouseDown(org.eclipse.swt.events.MouseEvent e) {
				tabsContainer.createHistoryStep();
			}
		});

		//add hair colors items to ColorChooserButton
		for (HairColoredSample hairColorImage : Loader.hairColorImages) {
			HairColorData hairColorData = hairColorImage.getHairColor();
			TableItem item = hairColorChooserButton.addTableItem(0);
			item.setData(hairColorData);
			item.setText(new String[] { "", messages.getString(hairColorData.getName()) });
			item.setImage(hairColorImage.getHairColorImage());
		}
	}

	/**
	 * This method initialize highlightsColorChooserButton
	 */
	private void createHighlightsColorChooserButton() {

		//TODO it should exist a method that generates text column width value by
		//getting the larger text's size (because of language files)
		int[] tabColumns = { 67, 200 };

		//grid data for the highlights color chooser button
		GridData gridDataHighlightsColorChooser = new GridData();
		gridDataHighlightsColorChooser.grabExcessHorizontalSpace = true;
		gridDataHighlightsColorChooser.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;

		//initialize the highlights color chooser button
		highlightsColorChooserButton = new ColorChooserButton(bottomComposite, tabColumns);
		highlightsColorChooserButton.setLayoutData(gridDataHighlightsColorChooser);

		//highlights color chooser listeners
		highlightsColorChooserButton.addTableSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				selectedHighlightsColor = (HairColorData)((TableItem)e.item).getData();
				redrawHairTabImage();
			}
		});
		highlightsColorChooserButton.addMouseListener(new MouseAdapter() {
			public void mouseDown(org.eclipse.swt.events.MouseEvent e) {
				tabsContainer.createHistoryStep();
			}
		});

		//add hair colors items to ColorChooserButton
		TableItem noneItem = highlightsColorChooserButton.addTableItem(0);
		noneItem.setText(new String[] { "", messages.getString("HAIR_TAB_NOHIGHLITES") });
		for (HairColoredSample hairColorImage : Loader.hairColorImages) {
			HairColorData hairColorData = hairColorImage.getHairColor();
			TableItem item = highlightsColorChooserButton.addTableItem(0);
			item.setData(hairColorData);
			item.setText(new String[] { "", messages.getString(hairColorData.getName()) });
			item.setImage(hairColorImage.getHairColorImage());
		}
	}

	@Override
	public void clear() {

		originalHighlitesImage = null;
		originalHairImage = null;
		originalShadowImage = null;
		finalImage = null;

		if (selectedHairButton != null) {
			selectedHairButton.setSelected(false);
			selectedHairButton = null;
		}
		selectedHairColor = null;
		selectedHighlightsColor = null;
		if (hairColorChooserButton != null) {
			hairColorChooserButton.deselect();
		}
		if (highlightsColorChooserButton != null) {
			highlightsColorChooserButton.deselect();
		}

		setBottomCompositeEnabled(false);

		selectedGender = femaleColorButton;
		femaleColorButton.setSelected(true);
		maleColorButton.setSelected(false);
		femaleColorButton.redraw();
		//select default combo
		hairTypesCombo.select(getDefaultHairType());
		Event evt = new Event();
		evt.widget = hairTypesCombo;
		hairTypesCombo.notifyListeners(SWT.Selection, evt);

	}

	/**
	 * Returns the default hair type that should be selected by default by the ComboBox
	 * or -1 in case of error
	 * @return the default hair type that should be selected by default by the ComboBox
	 */
	private int getDefaultHairType() {
		String defaultType = messages.getString(messages.getString("HAIR_DEFAULT_SELECTED_TYPE"));
		for (int i = 0; i < hairTypesCombo.getItems().length; i++) {
			String comboType = hairTypesCombo.getItems()[i];
			if (comboType.equals(defaultType)) {
				return i;
			}
		}
		return -1; //error occurred
	}

	@Override
	public void drawCurrentLayer(Graphics2D g2d) {
		if (HAIR_VISIBILITY == true && finalImage != null) {
			int imageWidthOffset = (finalImage.getWidth() - tabsContainer.RESIZED_IMAGE_WIDTH) / 2;
			int imageHeightOffset = (finalImage.getHeight() - tabsContainer.RESIZED_IMAGE_HEIGHT) / 2;
			g2d.drawImage(finalImage, hairTabImageOffset.x - imageWidthOffset, hairTabImageOffset.y - imageHeightOffset, null);
		}
	}

	@Override
	public Object getCurrentState() {
		return new HairHistoryStep(hairTabImageOffset,selectedHairButton,hairColorChooserButton.getSelectionIndex(), highlightsColorChooserButton.getSelectionIndex(), sizeScale.getSelection());
	}

	@Override
	public BufferedImage getLayerImage() {
		BufferedImage finalImage = null;
		if (HAIR_VISIBILITY == true && this.finalImage != null) {
			finalImage = new BufferedImage(tabsContainer.RESIZED_IMAGE_WIDTH, tabsContainer.RESIZED_IMAGE_HEIGHT, BufferedImage.TYPE_INT_ARGB);
			Graphics2D g2d = finalImage.createGraphics();
			drawCurrentLayer(g2d);
		}
		return finalImage;
	}

	@Override
	public void setCurrentState(Object o) {
		if (o instanceof HairHistoryStep) {
			HairHistoryStep hs = (HairHistoryStep) o;

			imageAreaRedrawAllowed = false;

			if (hs.selectedHairButton != null) {

				hairTabImageOffset = hs.finalImageOffset;

				String compositeGender = (String) hs.selectedHairButton.getParent().getData("gender");

				Event genderEvt = new Event();
				genderEvt.widget = compositeGender.equals(femaleColorButton.getData("gender"))?femaleColorButton:maleColorButton;
				selectedGender.setSelected(false);
				selectedGender = ((ColorButton)genderEvt.widget);
				selectedGender.setSelected(true);
				selectedGender.redraw();

				Integer comboIndex = (Integer) hs.selectedHairButton.getParent().getData("comboIndex");
				hairTypesCombo.select(comboIndex);

				originalHairImage = null;
				originalHighlitesImage = null;
				originalShadowImage = null;

				sizeScale.setSelection(hs.hairSizeScaleValue);
				highlightsColorChooserButton.select(hs.highlightsColorChooserIndex);
				hairColorChooserButton.select(hs.hairColorChooserIndex);


				Event evt = new Event();
				evt.widget = hs.selectedHairButton;
				hs.selectedHairButton.notifyListeners(SWT.MouseUp, evt);
				changeHairButtonsComposite(selectedHairButton.getParent());
				adjustScrollPosition(selectedHairButton);

			} else {
				clear();
			}

			imageAreaRedrawAllowed = true;
		}
	}

	/**
	 * Change the mouse cursor to HAND_CURSOR if there is a hair displayed
	 * @param e the mouse event
	 */
	private void setHandCursor(MouseEvent e) {
		if (finalImage != null) {
			e.getComponent().setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
		} else {
			e.getComponent().setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
		}
	}

	@Override
	public java.awt.event.MouseListener mouseListener() {
		if (mouseListener == null) {
			mouseListener = new java.awt.event.MouseListener() {

				//unimplemented methods
				public void mouseClicked(MouseEvent e) {}
				public void mouseExited(MouseEvent e) {}

				public void mouseEntered(MouseEvent e) {
					if (mouseIsDragged) {
						e.getComponent().setCursor(new java.awt.Cursor(java.awt.Cursor.MOVE_CURSOR));
					} else {
						setHandCursor(e);
					}
				}

				public void mousePressed(MouseEvent e) {
					if (finalImage == null) { //if there is no hair nothing to do in this method
						return ;
					}
					tabsContainer.getDisplay().syncExec(new Runnable() {
						public void run() {
							tabsContainer.createHistoryStep();
						}
					});
					startX = e.getX();
					startY = e.getY();
					oldImageOffset = new java.awt.Point(hairTabImageOffset);
					e.getComponent().setCursor(new java.awt.Cursor(java.awt.Cursor.MOVE_CURSOR));
				}

				public void mouseReleased(MouseEvent e) {
					mouseIsDragged = false;
					setHandCursor(e);
				}
			};
		}
		return mouseListener; 
	}

	public MouseMotionListener mouseMotionListener() {
		if (mouseMotionListener == null) {
			mouseMotionListener = new MouseMotionListener() {

				//unimplemented method
				public void mouseMoved(MouseEvent e) {}

				public void mouseDragged(MouseEvent e) {
					if (finalImage == null) { //if there is no hair nothing to do in this method
						return ;
					}
					mouseIsDragged = true;
					hairTabImageOffset.x =  e.getX() - startX + oldImageOffset.x;
					hairTabImageOffset.y =  e.getY() - startY + oldImageOffset.y;
					if (imageAreaRedrawAllowed) {
						tabsContainer.redrawImageArea(thisHairTab);
					}
				}
			};
		}
		return mouseMotionListener;
	}

	/**
	 * Indicate whether the hair tab should draw it's hair image in the drawing area
	 * @param value true for drawing the hair image, false otherwise
	 */
	public void setHairIsVisible(boolean value) {
		HAIR_VISIBILITY = value;
	}

	public void originalImageChanged() {
		// TODO Auto-generated method stub
	}

	public void disposeData() {
		finalImage = null;
		originalHairImage = null;
		originalHighlitesImage = null;
		originalShadowImage = null;
	}
}